------------------------------------------------------------------------------
-- fsltimer - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2008 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          fsltimer
-- Version:           1.00.a
-- Description:       Example FSL core (VHDL).
-- Date:              Fri Feb 10 15:13:05 2012 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library unisim;
use unisim.vcomponents.all;

-------------------------------------------------------------------------------------
--
--
-- Definition of Ports
-- FSL_Clk             : Synchronous clock
-- FSL_Rst           : System reset, should always come from FSL bus
-- FSL_S_Clk       : Slave asynchronous clock
-- FSL_S_Read      : Read signal, requiring next available input to be read
-- FSL_S_Data      : Input data
-- FSL_S_CONTROL   : Control Bit, indicating the input data are control word
-- FSL_S_Exists    : Data Exist Bit, indicating data exist in the input FSL bus
-- FSL_M_Clk       : Master asynchronous clock
-- FSL_M_Write     : Write signal, enabling writing to output FSL bus
-- FSL_M_Data      : Output data
-- FSL_M_Control   : Control Bit, indicating the output data are contol word
-- FSL_M_Full      : Full Bit, indicating output FSL bus is full
--
-------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- Entity Section
------------------------------------------------------------------------------

entity fsltimer is
    generic (
        TIMER_QUANTUM               : integer := 1
    );
    port (
		FSL_Clk	                    : in  std_logic;
		FSL_Rst	                    : in  std_logic;

		Interrupt                   : out std_logic;

		CPU_FSL1_OUT_Clk	        : in  std_logic;
		CPU_FSL1_OUT_Read	        : in  std_logic;
		CPU_FSL1_OUT_Data	        : out std_logic_vector(0 to 31);
		CPU_FSL1_OUT_Control	    : out std_logic;
		CPU_FSL1_OUT_Exists	        : out std_logic;

		CPU_FSL1_IN_Clk	            : in  std_logic;
		CPU_FSL1_IN_Write	        : in  std_logic;
		CPU_FSL1_IN_Data	        : in  std_logic_vector(0 to 31);
		CPU_FSL1_IN_Control	        : in  std_logic;
		CPU_FSL1_IN_Full	        : out std_logic
	);

attribute SIGIS : string; 
attribute SIGIS of FSL_Clk : signal is "Clk"; 

end fsltimer;


architecture SPM of fsltimer is

   constant SUBTIMER_MAX : Natural := TIMER_QUANTUM;

   signal activate                  : std_logic;
   signal trigger_value             : std_logic_vector(31 downto 0);
   signal timer_value               : std_logic_vector(31 downto 0);
   signal subtimer                  : Natural range 0 to SUBTIMER_MAX;


begin
    CPU_FSL1_IN_Full <= '0';
    CPU_FSL1_OUT_Data <= timer_value;
    CPU_FSL1_OUT_Exists <= '1';
    CPU_FSL1_OUT_Control <= '0';

    process (FSL_Clk) is
    begin
        if FSL_Rst = '1' then
            timer_value <= (others => '0');
            trigger_value <= (others => '0');
            subtimer <= 0;
            activate <= '0';

        elsif FSL_Clk'event and FSL_Clk = '1' then 
            Interrupt <= '0';
            if activate = '1' then
                if trigger_value <= timer_value then
                    Interrupt <= '1';
                end if;
                if subtimer = (TIMER_QUANTUM - 1) then
                    timer_value <= timer_value + 1;
                    subtimer <= 0;
                else
                    subtimer <= subtimer + 1;
                end if;
            end if;

            if (CPU_FSL1_IN_Write and CPU_FSL1_IN_Control) = '1' then
                if CPU_FSL1_IN_Data = x"ffffffff" then
                    -- software triggers reset
                    timer_value <= (others => '0');
                    trigger_value <= (others => '0');
                    subtimer <= 0;
                    activate <= '0';
                else
                    activate <= '1';
                    trigger_value <= CPU_FSL1_IN_Data;
                end if;
            end if;
        end if;
    end process;

end architecture SPM;


